import React, { useMemo } from 'react';
import { RawJSONVis, NodeLinkVis, PaohVis, SemanticSubstrates, TableVis } from '@graphpolaris/shared/lib/vis';
import { useAppDispatch, useGraphQueryResult, useQuerybuilderGraph, useVisualizationState } from '@graphpolaris/shared/lib/data-access';
import { LoadingSpinner } from '@graphpolaris/shared/lib/components/LoadingSpinner';
import { Visualizations, setActiveVisualization } from '@graphpolaris/shared/lib/data-access/store/visualizationSlice';
import { ArrowDropDown } from '@mui/icons-material';

export const VisualizationPanel = () => {
  const vis = useVisualizationState();
  const graphQueryResult = useGraphQueryResult();
  const query = useQuerybuilderGraph();
  const dispatch = useAppDispatch();

  const visualizationComponent = useMemo(() => {
    switch (vis.activeVisualization) {
      case Visualizations.Table:
        return (
          <div id={Visualizations.Table} className="tabContent w-full h-full">
            <TableVis showBarplot={true} />
          </div>
        );
      case Visualizations.NodeLink:
        return (
          <div id={Visualizations.NodeLink} className="tabContent w-full h-full">
            <NodeLinkVis />
          </div>
        );
      case Visualizations.RawJSON:
        return (
          <div id={Visualizations.RawJSON} className="tabContent w-full h-full">
            <RawJSONVis />
          </div>
        );
      case Visualizations.Paohvis:
        return (
          <div id={Visualizations.Paohvis} className="tabContent w-full h-full">
            <PaohVis rowHeight={30} hyperedgeColumnWidth={30} gapBetweenRanges={3} />
          </div>
        );
      default:
        return null;
    }
  }, [graphQueryResult, vis]);

  return (
    <div className="vis-panel h-full w-full overflow-y-auto" style={graphQueryResult.nodes.length === 0 ? { overflow: 'hidden' } : {}}>
      <h1>
        <span className="mr-2">Visualization Panel</span>
        <div className="dropdown">
          <label
            tabIndex={0}
            className="text-sm s-1 bg-slate-100 hover:bg-slate-200 transition-colors duration-300 rounded cursor-pointer px-2 py-1"
          >
            {vis.activeVisualization}
            <ArrowDropDown />
          </label>
          <ul tabIndex={0} className="dropdown-content z-[1] menu p-2 shadow bg-base-100 rounded-box w-52">
            <li onClick={() => dispatch(setActiveVisualization(Visualizations.Table))}>
              <a>Table</a>
            </li>
            <li onClick={() => dispatch(setActiveVisualization(Visualizations.NodeLink))}>
              <a>Node Link</a>
            </li>
            <li onClick={() => dispatch(setActiveVisualization(Visualizations.Paohvis))}>
              <a>PaohVis</a>
            </li>
            <li onClick={() => dispatch(setActiveVisualization(Visualizations.RawJSON))}>
              <a>JSON Structure</a>
            </li>
          </ul>
        </div>
      </h1>
      <div className="h-[calc(100%-2rem)]">
        {graphQueryResult.queryingBackend && (
          <div className="w-full h-full flex flex-col items-center justify-center overflow-hidden">
            <LoadingSpinner>Querying backend...</LoadingSpinner>
          </div>
        )}
        {!graphQueryResult.queryingBackend && graphQueryResult.nodes.length === 0 ? (
          <div className="w-full h-full flex flex-col items-center justify-center">
            <p>No data available to be shown</p>
            {query.nodes.length > 0 ? <p>Query resulted in empty dataset</p> : <p>Query for data to visualize</p>}
          </div>
        ) : (
          <div className="w-full h-full">{visualizationComponent}</div>
        )}
      </div>
    </div>
  );
};
