import React, { useEffect, useRef, useState } from 'react';
import {
  AddDatabaseRequest,
  databaseNameMapping,
  databaseProtocolMapping,
  useAppDispatch,
  useDatabaseAPI,
  DatabaseInfo,
} from '@graphpolaris/shared/lib/data-access';
import { RequiredInput } from '@graphpolaris/shared/lib/components/forms/requiredinput';
import { ErrorOutline } from '@mui/icons-material';
import { Dialog } from '@graphpolaris/shared/lib/components/Dialog';
import { addError } from '@graphpolaris/shared/lib/data-access/store/configSlice';

interface Connection {
  updating: boolean;
  status: null | string;
  verified: boolean | null;
}

const DEFAULT_DB: AddDatabaseRequest = {
  name: '',
  internal_database_name: '',
  url: '',
  port: 7687,
  protocol: '',
  username: '',
  password: '',
  type: 0,
};

export const SettingsForm = (props: { onClose(): void; open: boolean; database: DatabaseInfo | null }) => {
  const dispatch = useAppDispatch();
  const ref = useRef<HTMLDialogElement>(null);
  const [state, setState] = useState<AddDatabaseRequest>(DEFAULT_DB);
  const api = useDatabaseAPI();
  const [hasError, setHasError] = useState({});
  const [connection, setConnection] = useState<Connection>({
    updating: false,
    status: null,
    verified: null,
  });

  useEffect(() => {
    const db = props.database;
    if (db) {
      setState({
        name: db.Name,
        internal_database_name: db.InternalDatabaseName,
        url: db.URL,
        port: db.Port,
        protocol: db.Protocol,
        username: db.Username,
        password: db.Password,
        type: db.Type,
      });
    }
  }, [props.database]);

  useEffect(() => {
    if (connection.verified) {
      api.GetAllDatabases().catch((e) => console.debug(e));
      closeDialog();
    }
  }, [connection.verified]);

  function handleInputChange(field: keyof AddDatabaseRequest, value: string) {
    if (field != 'port' && field != 'type') {
      setState((prevState) => ({ ...prevState, [field]: value }));
    }
  }

  async function handleSubmit() {
    setConnection(() => ({
      updating: true,
      status: 'Updating database credentials',
      verified: null,
    }));

    if (props.database != null) {
      try {
        api
          .UpdateDatabase({
            name: props.database.Name,
            internal_database_name: state.internal_database_name,
            password: state.password,
            port: state.port,
            protocol: state.protocol,
            type: state.type,
            url: state.url,
            username: state.username,
          })
          .then(() => {
            setConnection(() => ({
              updating: false,
              status: 'Worked',
              verified: true,
            }));
          })
          .catch((e) => {
            dispatch(addError('Database updating went wrong'));
            setConnection((prevState) => ({
              ...prevState,
              updating: false,
              status: 'Database updating went wrong',
              verified: false,
            }));
          });
      } catch (e) {
        dispatch(addError('An error occured'));
        setConnection((prevState) => ({
          updating: false,
          status: 'An error occured',
          verified: false,
        }));
      }
    }
  }

  function handlePortChanged(port: string): void {
    if (!isNaN(Number(port))) setState({ ...state, port: Number(port) });
  }

  function closeDialog(): void {
    setConnection({
      updating: false,
      status: null,
      verified: null,
    });
    setState(DEFAULT_DB);
    props.onClose();
    ref.current?.close();
  }

  return (
    <Dialog open={props.open} onClose={props.onClose}>
      <div className="flex justify-between align-center">
        <h1 className="card-title">Update Credentials for {state.name} Database</h1>
      </div>

      <>
        <div className="form-control w-full ">
          <label className="label">
            <span className="label-text">Name of database</span>
          </label>
          <input type="text" className={`input input-bordered w-full`} value={state.name} disabled={true} />
        </div>

        <RequiredInput
          errorText="This field is required"
          label="Internal database name"
          placeHolder="internal_database_name"
          value={state.internal_database_name}
          validate={(v) => {
            setHasError({ ...hasError, internal_database_name: v.length === 0 });
            return v.length > 0;
          }}
          onChange={(value) => handleInputChange('internal_database_name', value)}
          type="text"
        />

        <div className="flex w-full gap-2">
          <div className="w-full">
            <label className="label">
              <span className="label-text">Database Type</span>
            </label>
            <select
              className="select select-bordered w-full max-w-xs"
              value={databaseNameMapping[state.type]}
              onChange={(event) => {
                setState({
                  ...state,
                  type: databaseNameMapping.indexOf(event.currentTarget.value),
                });
              }}
            >
              {databaseNameMapping.map((dbName) => (
                <option value={dbName} key={dbName}>
                  {dbName}
                </option>
              ))}
            </select>
          </div>
          <div className="w-full">
            <label className="label">
              <span className="label-text">Database Protocol</span>
            </label>
            <select
              className="select select-bordered w-full max-w-xs"
              value={state.protocol}
              onChange={(event) => {
                setState({
                  ...state,
                  protocol: event.currentTarget.value,
                });
              }}
            >
              {databaseProtocolMapping.map((protocol) => (
                <option value={protocol} key={protocol}>
                  {protocol}
                </option>
              ))}
            </select>
          </div>
        </div>

        <div className="flex w-full gap-2">
          <RequiredInput
            errorText="This field is required"
            label="Hostname/IP"
            placeHolder="neo4j"
            value={state.url}
            validate={(v) => {
              setHasError({ ...hasError, url: v.length === 0 });
              return v.length > 0;
            }}
            onChange={(value) => handleInputChange('url', value)}
            type="hostname"
          />

          <RequiredInput
            errorText="Must be between 1 and 9999"
            label="Port"
            placeHolder="neo4j"
            value={state.port}
            validate={(v) => {
              setHasError({ ...hasError, port: !(v <= 9999 && v > 0) });
              return v <= 9999 && v > 0;
            }}
            onChange={(value) => handlePortChanged(value)}
            type="port"
          />
        </div>

        <div className="flex w-full gap-2">
          <RequiredInput
            errorText="This field is required"
            label="Username"
            placeHolder="username"
            value={state.username}
            validate={(v) => {
              setHasError({ ...hasError, username: v.length === 0 });
              return v.length > 0;
            }}
            onChange={(value) => handleInputChange('username', value)}
            type="text"
          />
          <RequiredInput
            errorText="This field is required"
            label="Password"
            placeHolder="password"
            value={state.password}
            validate={(v) => {
              setHasError({ ...hasError, password: v.length === 0 });
              return v.length > 0;
            }}
            onChange={(value) => handleInputChange('password', value)}
            type="password"
          />
        </div>

        {!(connection.status === null) && (
          <div className={`flex flex-col justify-center items-center`}>
            <div className="flex justify-center items-center">
              {connection.verified === false && <ErrorOutline className="text-slate-400" />}
              <p className="font-light text-sm	text-slate-400	">{connection.status}</p>
            </div>
            {connection.verified === null && <progress className="progress w-56"></progress>}
          </div>
        )}
        <div className="card-actions w-full justify-center items-center">
          <button
            className={`btn btn-primary ${Object.values(hasError).some((e) => e === true) ? 'btn-disabled' : ''}`}
            type="button"
            disabled={connection.updating || Object.values(hasError).some((e) => e === true)}
            onClick={(event) => {
              event.preventDefault();
              handleSubmit();
            }}
          >
            {connection.updating ? 'Updating...' : 'Update'}
          </button>

          <button
            className="btn btn-outline"
            onClick={(e) => {
              e.preventDefault();
              closeDialog();
            }}
          >
            Cancel
          </button>
        </div>
      </>
    </Dialog>
  );
};
